#include <PubSubClient.h>
#include <SPI.h>
#include <Ethernet.h>
#include <SimpleDHT.h>
#include <DHT.h>




byte mac[] = {0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED};
IPAddress dnServer(192, 168, 1, 1);
IPAddress gateway(192, 168, 1, 1);
IPAddress subnet(255, 255, 255, 0);
IPAddress ip(192, 168, 1, 210);

const char* mqtt_server="192.168.1.200";

EthernetClient ethClient;
PubSubClient mqttClient(ethClient);

#define ENCHUFE 6
#define DHTTYPE DHT22   // DHT 22  (AM2302), AM2321
#define LIGHT A8
#define RELELUZ 7
float hum=0, temp=0;
unsigned long previousMillis = 0 ; // Almacena el valor de tiempo en la ultima lectura
const long interval = 30000; // Intervalo entre lecturas del sensor
int normalizedValue;
int estadoluz;
const int DHTPin = 53;
DHT dht(DHTPin, DHTTYPE);
const int PIRPin= 49;
int estadoPIR=5;
 
/*
Measuring AC Current Using ACS712
*/
const int sensorIn = A0;
int mVperAmp = 185; // use 185 for 5A Module, 100 for 20A Module and 66 for 30A Module
double Voltage = 0;
double VRMS = 0;
double AmpsRMS = 0;


// -----------------Leemos MQTT------------------

void callback(char* topic, byte* payload, unsigned int length) {
if (strcmp ("myhome/bedroom/switch1", topic) == 0){
 // Defaults to QoS 0 subscription:
  Serial.print("Mensaje recibido [");
  Serial.print(topic);
  Serial.print("] ");
  for (int i=0;i<length;i++) {
    char receivedChar = (char)payload[i];
    Serial.print(receivedChar);
 if (receivedChar == '1')
  digitalWrite(ENCHUFE, LOW);
  if (receivedChar == '0')
   digitalWrite(ENCHUFE, HIGH);
  }
  Serial.println();
}
if (strcmp ("myhome/bedroom/rele", topic) == 0){
 // Defaults to QoS 0 subscription:
  Serial.print("Mensaje recibido [");
  Serial.print(topic);
  Serial.print("] ");
  for (int i=0;i<length;i++) {
    char receivedChar = (char)payload[i];
    Serial.print(receivedChar);


// -----------------Función para controlar el estado del relé------------------

int releStatus = digitalRead(RELELUZ);

    //probamos con estadoluz************************************************************************************************
if (receivedChar == '0') estadoluz=0;

 if (receivedChar == '0' and estadoluz==1 and releStatus==0)
  digitalWrite(RELELUZ, HIGH);
  if (receivedChar == '1' and estadoluz==0 and releStatus==1)
   digitalWrite(RELELUZ, LOW);
   if (receivedChar == '0' and estadoluz==1 and releStatus==1)
  digitalWrite(RELELUZ, LOW);
  if (receivedChar == '1' and estadoluz==0 and releStatus==0)
   digitalWrite(RELELUZ, HIGH);
   
  Serial.println(releStatus);

  }
}
}
  // -----------------Función para reconectar en caso de caída-------------------------------

void reconnect() {
  while (!mqttClient.connected()) {
    Serial.print("Intentando conectarse...");
    if (mqttClient.connect("Arduino")) {
      Serial.println("conectado");
      mqttClient.subscribe("ledStatus");
    } else {
      Serial.print("falla, rc=");
      Serial.print(mqttClient.state());
      Serial.println(" reintentaremos en 5 segundos");
      delay(5000);
    }
  }
}

 
 // --------Función para lectura del DHT22 y del sensor de luz-----------------------------------------


void readDHT22(){ 
 //Serial.println("Reading data from DHT22...");
 // Reading temperature for humidity takes about 250 milliseconds!
 
 // Sensor readings may also be up to 2 seconds ’old’ (it’s a very slow sensor)

dht.begin();
 
 // Wait a few seconds between measurements.
 delay(2000);

  // Reading temperature or humidity takes about 250 milliseconds!
  hum = dht.readHumidity();
  temp = dht.readTemperature();

  if (isnan(hum) || isnan(temp)) {
   // Serial.println("Failed to read from DHT sensor!");
    return;
  }
  /*
  Serial.print("Humidity: ");
  Serial.print(hum);
  Serial.print(" %\t");
  Serial.print("Temperature: ");
  Serial.print(temp);
  Serial.print(" *C ");
*/

int value = analogRead(LIGHT);
normalizedValue = map(value, 0, 1023, 0, 99);
}

// -----------------Publicamos en MQTT los valores periódicos------------------

void mqttPublish(){
  unsigned long currentMillis = millis();
char temp_str[3], hum_str[3], lux_str[3];
if (currentMillis - previousMillis >= interval){
previousMillis = currentMillis;

 readDHT22(); //leemos los datos del sensor
  
// Convierte los valores de temperatura y humedad a cadena de caracteres
 dtostrf(temp,2,1,temp_str);
 dtostrf(hum,2,1,hum_str);
 dtostrf(normalizedValue,2,1,lux_str);
  
 // Envio de datos con MQTT
 //Serial.println ("DTH sensor read and transmitted");


 /*Serial.print (" Estado MQTT - ");
       Serial.print(mqttClient.state());
 Serial.println (" - 0 es conectado");
*/


 mqttClient.publish("myhome/bedroom/temperature", temp_str);
 mqttClient.publish("myhome/bedroom/humidity", hum_str);
 mqttClient.publish("myhome/bedroom/lux", lux_str);

 
 // Guarda el valor de la ultima lectura
 previousMillis = millis();
}
}

// -----------------Función para establecer nivel de corriente en el sensor ACS712, lee la función getVPP()------------------

void Luz(){
 Voltage = getVPP();
 VRMS = (Voltage/2.0) *0.707; 
 AmpsRMS = (VRMS * 1000)/mVperAmp;
 Serial.print(AmpsRMS);
 Serial.println(" Amps RMS");


if (AmpsRMS>0.15 and estadoluz==0){
  estadoluz=1;
   mqttClient.publish("myhome/bedroom/luz", "1");
  }  
 if (AmpsRMS<0.15 and estadoluz==1){
  estadoluz=0;
  mqttClient.publish("myhome/bedroom/luz", "0");
  }
  }

// -----------------Función para leer el sensor de movimiento------------------

void Pir(){
   int value= digitalRead(PIRPin);

  if (value == HIGH and estadoPIR!=1)
  {
     mqttClient.publish("myhome/bedroom/PIR", "OPEN");
  estadoPIR=1;
  }
  if (value == LOW and estadoPIR!=0){
    mqttClient.publish("myhome/bedroom/PIR", "CLOSED");
  estadoPIR=0;
  }
}

// -----------------Función para leer el sensor ACS712------------------

float getVPP(){
  float result;
  
  int readValue;             //value read from the sensor
  int maxValue = 0;          // store max value here
  int minValue = 1024;          // store min value here
  
   uint32_t start_time = millis();
   while((millis()-start_time) < 1000) //sample for 1 Sec
   {
       readValue = analogRead(sensorIn);
       // see if you have a new maxValue
       if (readValue > maxValue) 
       {
           /*record the maximum sensor value*/
           maxValue = readValue;
       }
       if (readValue < minValue) 
       {
           /*record the maximum sensor value*/
           minValue = readValue;
       }
   }
   
   // Subtract min from max
   result = ((maxValue - minValue) * 5.0)/1024.0;
      
   return result;
 }

 
void setup() {
  Serial.begin(9600);
  mqttClient.setServer(mqtt_server, 1883);
  mqttClient.setCallback(callback);

  pinMode(ENCHUFE, OUTPUT);
  pinMode(RELELUZ, OUTPUT);
  pinMode(PIRPin, INPUT);

}
 
void loop() {
  if (!mqttClient.connected()) {
    // initialize the ethernet device
  Ethernet.begin(mac, ip, dnServer, gateway, subnet);
  //print out the IP address
  Serial.print("IP = ");
  Serial.println(Ethernet.localIP()); 
    reconnect(); //Función para reconectar en caso se caída
  mqttClient.subscribe("myhome/bedroom/#");

  }
  //Lanzamos las diferentes funciones dependientes
  Luz(); 
  Pir();
  mqttPublish(); // Publica los datos con el protocolo MQTT
 delay (300) ;

   mqttClient.loop();
    
   
}

